import threading
from time import sleep
#from GH.dht11 import DHT11 #Current config is using two analog sensors

class GHTemperature(threading.Thread):
    """This class GHFan provides a services to control the IoT Greenhouse fan
    """
    
    _analogTempSensor_in = None
    _analogTempSensor_out = None
#    _digitalTempSensor = None

    inside_temp_C = 0
    inside_temp_F = 0
    outside_temp_C = 0
    outside_temp_F = 0
 #   humidity = 0
    reading_is_valid = False
    last_valid_read = 0

    def __init__(self, analog_temp_channel_in, analog_temp_channel_out):
        self._digitalTempSensor = DHT11(digital_pin)
        self._analogTempSensor = analog_temp_channel
        threading.Thread.__init__(self)
        self.daemon = True
        self.start()

    def run(self):
        #get inside temp and humidity
        result = self._digitalTempSensor.read()
        if result.is_valid():
            self.reading_is_valid = True
            self.last_valid_read = datetime.datetime.now()
            self.inside_temp_C = result.temperature
            self.inside_temp_F = convert_C_to_F(self.inside_temp_C)
            self.humidity = result.humidity
        #get outside temp
        analog_value = self._analogTempSensor.get_value()
        self.outside_temp_C = self._convert_to_temp(analog_value, 1)
        self.outside_temp_F = self.convert_C_to_F(self.outside_temp_C) 
        
        sleep(1)


    def convert_C_to_F(self, temp_C):
        return temp_C * 9/5 +32

    def convert_F_to_C(self, temp_F):
        return (temp_F - 32) * 5/9

    # Function to calculate temperature from
    # TMP36 data, rounded to specified
    # number of decimal places.
    def _convert_to_temp(self, data, places):
 
      # ADC Value
      # (approx)  Temp  Volts
      #    0      -50    0.00
      #   78      -25    0.25
      #  155        0    0.50
      #  233       25    0.75
      #  310       50    1.00
      #  465      100    1.50
      #  775      200    2.50
      # 1023      280    3.30
 
      temp = ((data * 330)/float(1023))-50
      temp = round(temp,places)
      return temp



import datetime
import pigpio
import configparser, os

from iot_gh.GHgpio import GHgpio
from iot_gh.GHAnalogService import GHAnalogService
from iot_gh.GHBuzzer import GHBuzzer
from iot_gh.GHFan import GHFan
from iot_gh.GHLamps import GHLamps
from iot_gh.GHServo import GHServo
from iot_gh.GHSwitches import GHSwitches
from iot_gh.GHTemperature import GHTemperature
from iot_gh.GHWebService import GHWebService
from iot_gh.IoTGreenhouse import IoTGreenhouse


class IoTGreenhouseService(object):
    """IoT Greenhouse service class for IoT Greenhouse
    
    Provides references to input and output objects.
    """
    _pi = None      #pigpio connection
    _url = None
    _servo_ccw_limit = None
    _servo_cw_limit = None
    
    version = None
    
    analog_service = None
    ain_pot = None
    ain_temp = None
    ain_light = None
    ain_aux = None
    buzzer = None
    fan = None
    lamps = None
    servo = None
    switches = None
    temperature = None
    web_service = None

    def __init__(self):

        self._pi = pigpio.pi()             # exit script if no connection
        if not self._pi.connected:
            print("ERROR: unable to connect to pigpio")
            exit()
        self._read_config()
        self._make_components()

    def _read_config(self):
        """Load configuration file from user iot_gh directory.
        """
        try:
            config = configparser.ConfigParser()
            config.read(["iot_gh_system.conf", os.path.expanduser("~/iot_gh/iot_gh_system.conf")],encoding="UTF8")
            self.version = config["IOT_GREENHOUSE"]["VERSION"]
            self._url = config["IOT_GREENHOUSE"]["URL"]
            self._servo_ccw_limit = int(config["IOT_GREENHOUSE"]["SERVO_CCW_LIMIT"])
            self._servo_cw_limit = int(config["IOT_GREENHOUSE"]["SERVO_CW_LIMIT"])
        except :
            raise Exception("Unable to load IoT Greenhouse Systesm Configuration.")
       
    def _make_components(self):
        """Makes component services for IoT Greenhouse Service.
        """
        self.analog = GHAnalogService(self._pi)
        self.buzzer = GHBuzzer(self._pi, GHgpio.BUZZER)
        self.fan = GHFan(self._pi, GHgpio.FAN)
        self.lamps = GHLamps(self._pi, GHgpio.RED_LED, GHgpio.WHITE_LED, GHgpio.RED_LED, GHgpio.DUAL_LED)
        self.servo = GHServo(self._pi, GHgpio.SERVO_PWM, self._servo_cw_limit, self._servo_ccw_limit)
        self.switches = GHSwitches(self._pi, GHgpio.SWITCH_PB, GHgpio.SWITCH_TOGGLE)
        self.temperature = GHTemperature(self.analog.aux, self.analog.temp)
        self.web_service = GHWebService(self._url)    
        
    def make_greenhouse(self, name):
        """Factory class to build IoT Greenhouse data object.
        """
        gh = IoTGreenhouse()
        gh.name = name
        
        #read greenhouse config values
        config = configparser.ConfigParser()
        config.read(["iot_gh.conf", os.path.expanduser("~/iot_gh/iot_gh.conf")],encoding="UTF8")
        gh.house_id = config["IOT_GREENHOUSE"]["HOUSE_ID"]
        gh.row_id = config["IOT_GREENHOUSE"]["ROW_ID"]
        gh.group_id = config["IOT_GREENHOUSE"]["GROUP_ID"]
     
        self.update_state(gh)

        return gh
    
    def update_state(self, gh):
        """Updates IoTGreenhouse object by reading all house 
        states and refreshing gh object.
        """
        try:
    
            gh.led_red_state = self.lamps.red.get_state()
            gh.led_white_state = self.lamps.white.get_state()
            gh.led_dual_state =  self.lamps.dual.get_state()
            gh.switch_pb_state = self.switches.push_button.get_state()
            gh.switch_toggle_state = self.switches.toggle.get_state()
            gh.fan_state =  self.fan.get_state()
            gh.servo_position = self.servo.get_value()
            gh.buzzer_state = self.buzzer.get_state()
            gh.ain_pot_raw = self.analog.pot.get_value()
            gh.ain_light_raw =  self.analog.light.get_value()
            gh.ain_aux_raw =  self.analog.aux.get_value()

            gh.temp_inside_C = self.temperature.get_inside_temp_C()
            gh.temp_inside_F = self.temperature.get_inside_temp_F()
            gh.temp_outside_C = self.temperature.get_outside_temp_C()
            gh.temp_outside_F = self.temperature.get_outside_temp_F()
            #gh.temp_humidity = self.temperature.humidity
            #gh.temp_reading_is_valid = self.temperature.reading_is_valid
            #gh.temp_last_valid_reading = self.temperature.last_valid_read
            
            gh.valid_state = True
            gh.last_update = datetime.datetime.now()
        except Exception as e:
            gh.message = e.args[0]
            gh.valid_state = False
            gh.last_update = None




import pigpio

class GHFan(object):
    """This class GHFan provides a services to control the IoT Greenhouse fan
    """
    
    FAN_ON = True
    FAN_OFF = False

    _pi = None
    _gpio = 0
        
    def __init__(self, pi, fan_gpio):
        """Constructor for class GHFan
        
        :param pi: reference to pigpio service
        :param fan_gpio: GPIO port for fan. Use GHgpio definitions.
        :returns: GHFan object
        """
        self._pi = pi
        self._gpio = fan_gpio
            
        self._pi.set_mode(self._gpio, pigpio.OUTPUT)        
        #Fan off
        self._pi.write(self._gpio, self.FAN_OFF)
        
    def on(self):
        self._pi.write(self._gpio, self.FAN_ON)
        
    def off(self):
        self._pi.write(self._gpio, self.FAN_OFF)
        
    def get_state(self):
        return self._pi.read(self._gpio)

    def is_on(self):
        return self.get_state() == self.FAN_ON

    def is_off(self):
        return self.get_state() == self.FAN_OFF
